@@ -2,22 +2,19 @@ package Net::ThreeScale::Client;
 
 use strict;
 use warnings;
-use vars qw($VERSION @ISA @EXPORT_OK %EXPORT_TAGS %QUEUE @QUEUE $QUEUE);
-use Exporter;
-use Data::Dumper;
-use Carp;
+use vars qw($VERSION @ISA @EXPORT_OK %EXPORT_TAGS);
 
+use Carp;
+use Data::Dumper;
+use Exporter;
+use HTTP::Tiny;
+use Net::ThreeScale::Response;
+use Try::Tiny;
+
+use URI::Escape::XS qw(uri_escape);
 use XML::Parser;
 use XML::Simple;
 
-use Ref::Util qw(is_arrayref is_hashref);
-use LWP::UserAgent;
-use URI::Escape;
-use HTTP::Request;
-use HTTP::Status;
-use HTTP::Request::Common;
-
-use Net::ThreeScale::Response;
 my $DEFAULT_USER_AGENT;
 
 use constant {
@@ -28,7 +25,7 @@ use constant {
 
 BEGIN {
 	@ISA         = qw(Exporter);
-	$VERSION     = "2.1.3";
+	$VERSION     = "2.1.5";
 	@EXPORT_OK   = qw();
 	%EXPORT_TAGS = (
 		'all' => \@EXPORT_OK,
@@ -44,14 +41,21 @@ sub new {
 
 	my $agent_string = $params->{user_agent} || $DEFAULT_USER_AGENT;
 
-	croak("provider_key is a required parameter")
-		unless $params->{provider_key};
+	croak("provider_key or service_token/service_id pair are required")
+		unless $params->{provider_key} xor ( $params->{service_token} && $params->{service_id});
 
 	my $self = {};
-	$self->{provider_key} = $params->{provider_key};
-	$self->{url}          = $params->{url} || 'http://su1.3scale.net';
+	$self->{provider_key}  = $params->{provider_key}  || undef;
+	$self->{service_token} = $params->{service_token} || undef;
+	$self->{service_id}    = $params->{service_id}    || undef;
+
+	$self->{url}          = $params->{url} || 'https://su1.3scale.net';
 	$self->{DEBUG}        = $params->{DEBUG};
-	$self->{ua}           = LWP::UserAgent->new( agent => $agent_string );
+	$self->{HTTPTiny}     = HTTP::Tiny->new(
+		'agent'      => $agent_string,
+		'keep_alive' => 1,
+		'timeout'    => 5,
+	);
 
 	return bless $self, $class;
 }
@@ -60,18 +64,20 @@ sub _authorize_given_url{
     my $self = shift;
     my $url = shift;
 
-    my $request = HTTP::Request::Common::GET($url);
-    $self->_debug( "start> sending request: ", $request->as_string );
+    $self->_debug( "start> sending GET request: ", $url );
 
-    my $response = $self->{ua}->request($request);
-    $self->_debug( "start> got response : ", $response->as_string );
+    my $response = $self->{HTTPTiny}->get($url);
+    $self->_debug( "start> got response : ", $response->{content} );
 
+    if (!$response->{success}){
+        return $self->_wrap_error($response);
+    }
     # HTTP 409 = Conflict
-    if ( not ( $response->is_success || $response->status_line =~ /409/)) {
+    if ($response->{status} == 409){
         return $self->_wrap_error($response);
     }
 
-    my $data = $self->_parse_authorize_response( $response->content() );
+    my $data = $self->_parse_authorize_response( $response->{content} );
 	
     if ($data->{authorized} ne "true") {
         
@@ -102,7 +108,9 @@ sub authorize {
 	die("app_id is required") unless defined($p->{app_id});
 
 	my %query = (
-		provider_key => $self->{provider_key},
+		(provider_key  => $self->{provider_key})x!!  $self->{provider_key},
+		(service_token => $self->{service_token})x!! $self->{service_token},
+		(service_id    => $self->{service_id})x!!    $self->{service_id},
 	);
 
 	while (my ($k, $v) = each(%{$p})) {
@@ -123,7 +131,9 @@ sub authrep {
 	die("user_key is required") unless defined($p->{user_key});
 
 	my %query = (
-		provider_key => $self->{provider_key},
+		(provider_key  => $self->{provider_key})x!!  $self->{provider_key},
+		(service_token => $self->{service_token})x!! $self->{service_token},
+		(service_id    => $self->{service_id})x!!    $self->{service_id},
 	);
 
 	while (my ($k, $v) = each(%{$p})) {
@@ -150,10 +160,12 @@ sub report {
 
 	die("transactions is a required parameter") unless defined($p->{transactions});
 	die("transactions parameter must be a list")
-		unless (is_arrayref($p->{transactions}));
+		unless (ref($p->{transactions}) eq 'ARRAY');
 
 	my %query = (
-		provider_key => $self->{provider_key},
+		(provider_key  => $self->{provider_key})x!!  $self->{provider_key},
+		(service_token => $self->{service_token})x!! $self->{service_token},
+		(service_id    => $self->{service_id})x!!    $self->{service_id},
 	);
 
 	while (my ($k, $v) = each(%{$p})) {
@@ -180,14 +192,13 @@ sub report {
 
 	my $url = $self->{url} . "/transactions.xml";
 
-	my $request = HTTP::Request::Common::POST($url, Content=>$content);
+	$self->_debug( "start> sending request: ", $url );
 
-	$self->_debug( "start> sending request: ", $request->as_string );
-	my $response = $self->{ua}->request($request);
+	my $response = $self->{HTTPTiny}->post_form($url, { Content=>$content });
 
-	$self->_debug( "start> got response : ", $response->as_string );
+	$self->_debug( "start> got response : ", $response->{content} );
 
-	if ( not $response->is_success ) {
+	if ( !$response->{success} ) {
 		return $self->_wrap_error($response);
 	}
 
@@ -206,16 +217,16 @@ sub _wrap_error {
 	my $error_code;
 	my $message;
 
-	eval { ( $error_code, $message ) = $self->_parse_errors( $res->content() ); };
-
-	if ($@) {
+	try {
+		( $error_code, $message ) = $self->_parse_errors( $res->{content});
+	} catch {
 		$error_code = TS_RC_UNKNOWN_ERROR;
 		$message    = 'unknown_error';
-	}
+	};
 
 	return Net::ThreeScale::Response->new(
 		success    => 0,
-		error_code => $error_code,    #
+		error_code => $error_code,
 		error_message      => $message
 	);
 }
@@ -261,7 +272,12 @@ sub _parse_errors {
 		}
 	);
 
-	eval { $parser->parse($body); };
+	try {
+		$parser->parse($body);
+	}
+	catch {
+		$errstring = $_;
+	};
 
 	return ( $errcode, $errstring );
 }
@@ -270,15 +286,11 @@ sub _parse_authorize_response {
 	my $self          = shift;
 	my $response_body = shift;
 
-	my $xml = new XML::Simple(ForceArray=>['usage_report']);
-
-	my $data = {};
-
 	if (length($response_body)) {
-		$data = $xml->XMLin($response_body);
+		my $xml = new XML::Simple(ForceArray=>['usage_report']);
+		return $xml->XMLin($response_body);
 	}
-
-	return $data;
+	return {};
 }
 
 sub _format_transactions {
@@ -291,13 +303,13 @@ sub _format_transactions {
 
 	for my $trans (@transactions) {
 		die("Transactions should be given as hashes")
-			unless(is_hashref($trans));
+			unless(ref($trans) eq 'HASH');
 
 		die("Transactions need an 'app_id'")
 			unless(defined($trans->{app_id}));
 
 		die("Transactions need a 'usage' hash")
-			unless(defined($trans->{usage}) and is_hashref($trans->{usage}));
+			unless(defined($trans->{usage}) and ref($trans->{usage}) eq 'HASH');
 
 		die("Transactions need a 'timestamp'")
 			unless(defined($trans->{app_id}));
@@ -318,7 +330,10 @@ sub _format_transactions {
 			$output .= $pref . "[usage][$k]=$v";
 		}
 
-		$output .= "&" . $pref . "[timestamp]=" . uri_escape($trans->{timestamp});
+		$output .= "&"
+			. $pref
+			. "[timestamp]="
+			. uri_escape($trans->{timestamp});
 
 		$transNumber += 1;
 	}
@@ -343,9 +358,13 @@ Net::ThreeScale::Client - Client for 3Scale.com web API version 2.0
 
  use Net::ThreeScale::Client;
  
- my $client = new Net::ThreeScale::Client(provider_key=>"my_assigned_provider_key", 
+ my $client = new Net::ThreeScale::Client(provider_key=>"my_assigned_provider_key",
                                         url=>"http://su1.3Scale.net");
- 
+
+ # Or initialize by service_token/service_id
+ # my $client = new Net::ThreeScale::Client(service_token=>"SERVICE_TOKEN",
+ #                                       service_id=>"SERVICE_ID");
+
  my $response = $client->authorize(app_id  => $app_id,
                                    app_key => $app_key);
           
@@ -400,10 +419,18 @@ Net::ThreeScale::Client - Client for 3Scale.com web API version 2.0
 
 =over 4
  
-=item provider_key 
+=item provider_key
 
 (required) The provider key used to identify you with the 3Scale service
 
+=item service_token
+
+(required) Service API key with 3scale (also known as service token).
+
+=item service_id
+
+(required) Service id. Required.
+
 =item url 
 
 (optional) The 3Scale service URL, usually this should be left to the 
@@ -464,7 +491,6 @@ which may appear in calls to Net::ThreeScale::Response::error_code
 
 The operation completed successfully 
 
-
 =item TS_RC_AUTHORIZE_FAILED
 
 The  passed provider key was invalid
@@ -478,7 +504,8 @@ An unspecified error occurred.  See the corresponding message for more detail.
 =head1 SUPPORT
 
 3scale support say,
-I<We do not have anyone in 3scale actively maintaining it, but we will certainly monitor pull requests and consider merging any useful contributions.>
+I<We do not have anyone in 3scale actively maintaining it, but we will
+certainly monitor pull requests and consider merging any useful contributions.>
 
 =head1 SEE ALSO 
 
@@ -492,7 +519,23 @@ Contains details of response contnet and values.
 
 =head1 AUTHOR 
 
-(c) Owen Cliffe 2008, Eugene Oden 2010. Collaborators: Dave Lambley, Ed Freyfogle and Marc Metten.
+(c) Owen Cliffe 2008, Eugene Oden 2010.
+
+=head1 CONTRIBUTORS
+
+=over
+
+=item *
+
+Dave Lambley
+
+=item *
+
+Ed Freyfogle
+
+=item *
+
+Marc Metten
 
 =head1 LICENSE
 
