@@ -1,5 +1,8 @@
 package Net::ThreeScale::Response;
 
+use warnings;
+use strict;
+
 sub new {
 	my $class = shift; 
 	my $self = {@_};
@@ -75,8 +78,9 @@ Returns a textual description of the error returned by the server.
 
 =head1 SEE ALSO
 
-Net::ThreeScale::Client
+L<Net::ThreeScale::Client>
  
 =head1 AUTHOR
   Owen Cliffe 
   Eugene Oden
+  Collaborators: Dave Lambley, Ed Freyfogle and Marc Metten.
